#pragma once

#include <thread>
#include <memory>

#include "clientapi.h"

class LolAutoAccept {
protected:
	struct Stage {
		Stage();
		virtual ~Stage();

		std::vector<uint32_t> champids; // not every stage has a champ
		bool enabled = false;
		uint32_t currentOffset = 0;
	};

	std::vector<Stage*> stages;

	bool shouldrun = false;
	std::thread lolaathread;

	std::shared_ptr<ClientAPI> clientapi;

	int64_t summonerid = -1;

public:
	enum class State {
		LOBBY = 0,
		PREPICK = 1,
		BAN = 2,
		PICK = 3,
		GAME = 4
	};

	LolAutoAccept();
	~LolAutoAccept();

	void setChamps(const std::vector<uint32_t>& champs, State s);
	void setEnabled(bool b, State s);

	bool init(); // returns true on success
	void run();
	void stop();

private:
	void stopJoinThread();
	void innerRun();
	void resetAllOffsets();

	uint32_t getChampOfState(State s);
	void nextChampOfState(State s);

	static bool isChampIntentofTeammate(uint32_t champid, const ClientAPI::ChampSelectSession& session);

	using ownactions_t = std::vector<ClientAPI::ChampSelectAction>;
	ownactions_t getOwnActions(int32_t cellid, const std::vector<ClientAPI::ChampSelectAction> actions);
	void prepickPhase(const ownactions_t& ownactions);
	void banPhase(const ownactions_t& ownactions, const ClientAPI::ChampSelectSession& session);
	void pickPhase(const ownactions_t& ownactions);
	void phase(const ownactions_t& ownactions, ClientAPI::ChampSelectActionType type, State s, bool complete, std::function<bool(uint32_t)> filter = {});
	void champSelect();
};